package com.gigachatwoot.connectors.chatwoot.client;

import com.gigachatwoot.connectors.chatwoot.models.ChatMessage;
import com.gigachatwoot.connectors.chatwoot.models.ChatwootMessagesResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.PropertySource;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.*;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Component
@RequiredArgsConstructor
@PropertySource("classpath:application.yml")
public class ChatwootClient {
    private final RestTemplate restTemplate = new RestTemplate();

    @Value("${chatwoot.api.access.token.agent.bot}")
    private String apiAccessTokenAgentBot;

    @Value("${chatwoot.api.access.token.user}")
    private String apiAccessTokenUser;

    @Value("${chatwoot.api.base.url}")
    private String baseUrl;

    public void sendMessageToChatwoot(Long accountId,
                                      Long conversationId,
                                      String messageContent) {
        String url = baseUrl + "/api/v1/accounts/" + accountId + "/conversations/" + conversationId + "/messages";

        HttpHeaders headers = new HttpHeaders();
        headers.set("Content-Type", "application/json");
        headers.set("api_access_token", apiAccessTokenAgentBot);

        Map<String, Object> requestBody = new HashMap<>();
        requestBody.put("content", messageContent);
        requestBody.put("message_type", "outgoing");
        requestBody.put("private", false);

        HttpEntity<Map<String, Object>> requestEntity = new HttpEntity<>(requestBody, headers);

        ResponseEntity<String> response = restTemplate.exchange(url, HttpMethod.POST, requestEntity, String.class);

        if (response.getStatusCode() != HttpStatus.OK) {
            log.warn("sendMessageToChatwoot: Failed to retrieve message history from Chatwoot: {}",
                response.getStatusCode());
        }
    }

    public List<ChatMessage> getMessageHistory(Long accountId, Long conversationId) {
        String url = baseUrl + "/api/v1/accounts/" + accountId + "/conversations/" + conversationId + "/messages";

        HttpHeaders headers = new HttpHeaders();
        headers.set("Content-Type", "application/json");
        headers.set("api_access_token", apiAccessTokenUser);

        HttpEntity<String> requestEntity = new HttpEntity<>(headers);

        ResponseEntity<ChatwootMessagesResponse> response = restTemplate.exchange(url, HttpMethod.GET,
            requestEntity, ChatwootMessagesResponse.class);

        if (response.getStatusCode() == HttpStatus.OK && response.getBody() != null) {
            return response.getBody().getPayload();
        } else {
            log.warn("getMessageHistory: Failed to retrieve message history from Chatwoot: {}",
                response.getStatusCode());
            return Collections.emptyList();
        }
    }

    public List<ChatMessage> sortAndFilterMessages(List<ChatMessage> messages) {
        // Find the index of the last message with message_type = 2
        int lastResolutionIndex = -1;
        for (int i = messages.size() - 1; i >= 0; i--) {
            if (messages.get(i).getMessageType() == 2) {
                lastResolutionIndex = i;
                break;
            }
        }

        // If a message with message_type = 2 is found, ignore all previous messages
        List<ChatMessage> filteredMessages;
        if (lastResolutionIndex != -1) {
            filteredMessages = messages.subList(lastResolutionIndex + 1, messages.size());
        } else {
            filteredMessages = messages;
        }

        // Filter and sort messages
        return filteredMessages.stream()
            .filter(msg -> msg.getMessageType() == 0 || msg.getMessageType() == 1)
            .sorted(Comparator.comparing(ChatMessage::getId))
            .collect(Collectors.toList());
    }


    public String formatMessagesForConsole(List<ChatMessage> messages) {
        StringBuilder sb = new StringBuilder();
        for (ChatMessage message : messages) {
            sb.append("ID: ").append(message.getId()).append("\n");
            sb.append("Content: ").append(message.getContent()).append("\n");
            sb.append("Message type: ").append(message.getMessageType() == 0 ? "Incoming" : "Outgoing").append("\n");
            sb.append("Sender type: ").append(message.getSender().getType()).append("\n");
            sb.append("-----------------------------\n");
        }
        return sb.toString();
    }

    public void assignConversationToAgent(Long accountId, Long conversationId, Long agentId) {
        String url = baseUrl + "/api/v1/accounts/" + accountId + "/conversations/" + conversationId + "/assignments ";

        HttpHeaders headers = new HttpHeaders();
        headers.set("Content-Type", "application/json");
        headers.set("api_access_token", apiAccessTokenUser);

        Map<String, Object> requestBody = new HashMap<>();
        requestBody.put("assignee_id", agentId);

        HttpEntity<Map<String, Object>> requestEntity = new HttpEntity<>(requestBody, headers);

        ResponseEntity<String> response = restTemplate.exchange(url, HttpMethod.POST, requestEntity, String.class);

        if (response.getStatusCode() != HttpStatus.OK) {
            log.warn("assignAgentToConversation: Failed to assign agent to conversation: {}",
                response.getStatusCode());
        }
    }

    public Long getRandomAvailableAgentId(Long accountId) {
        String url = baseUrl + "/api/v1/accounts/" + accountId + "/agents";

        HttpHeaders headers = new HttpHeaders();
        headers.set("Content-Type", "application/json");
        headers.set("api_access_token", apiAccessTokenUser);

        HttpEntity<String> requestEntity = new HttpEntity<>(headers);

        ResponseEntity<List<Map<String, Object>>> response = restTemplate.exchange(url, HttpMethod.GET,
            requestEntity, new ParameterizedTypeReference<>() {
            });

        if (response.getStatusCode() == HttpStatus.OK && response.getBody() != null) {
            List<Map<String, Object>> agents = response.getBody();
            if (!agents.isEmpty()) {
                Collections.shuffle(agents, new Random());
                return ((Number) agents.get(0).get("id")).longValue();
            }
        } else {
            log.warn("getFirstAvailableAgentId: Failed to retrieve agents list: {}",
                response.getStatusCode());
        }
        return null;
    }
}
